from pathlib import Path

from polylith import log, repo


template = """\
# A Python Polylith repo

## Docs
The official Polylith documentation:
[high-level documentation](https://polylith.gitbook.io/polylith)

A Python implementation of the Polylith tool:
[python-polylith](https://github.com/DavidVujic/python-polylith)
"""


logger = log.getLogger()


def create_workspace_readme(path: Path, namespace: str) -> None:
    fullpath = path / repo.readme_file

    if fullpath.exists():
        logger.info(f"A {repo.readme_file} already exists. Skipping this step.")
        return

    with fullpath.open("w", encoding="utf-8") as f:
        f.write(template.format(namespace=namespace))



from pathlib import Path

import tomlkit
from polylith import repo


def _load_workspace_config(path: Path) -> tomlkit.TOMLDocument:
    fullpath = path / repo.workspace_file

    content = fullpath.read_text()

    return tomlkit.loads(content)


def get_namespace_from_config(path: Path) -> str:
    toml: dict = _load_workspace_config(path)

    return toml["tool"]["polylith"]["namespace"]


def get_git_tag_pattern_from_config(path: Path) -> str:
    toml: dict = _load_workspace_config(path)

    return toml["tool"]["polylith"]["git_tag_pattern"]


def is_test_generation_enabled(path: Path) -> bool:
    toml: dict = _load_workspace_config(path)

    enabled = toml["tool"]["polylith"]["test"]["enabled"]
    return bool(enabled)


def get_theme_from_config(path: Path) -> str:
    toml: dict = _load_workspace_config(path)

    return toml["tool"]["polylith"]["structure"]["theme"] or "tdd"


def get_brick_structure_from_config(path: Path) -> str:
    theme = get_theme_from_config(path)

    if theme == "loose":
        return "{brick}/{namespace}/{package}"

    return "{brick}/{package}/src/{namespace}/{package}"


def get_tests_structure_from_config(path: Path) -> str:
    theme = get_theme_from_config(path)

    if theme == "loose":
        return "test/{brick}/{namespace}/{package}"

    return "{brick}/{package}/test/{namespace}/{package}"



from polylith.project.create import create_project
from polylith.project.get import get_project_names, get_packages_for_projects
from polylith.project.parser import parse_package_paths

__all__ = [
    "create_project",
    "get_project_names",
    "get_packages_for_projects",
    "parse_package_paths",
]



from polylith.info.collect import get_bricks_in_projects
from polylith.info.report import print_bricks_in_projects, print_workspace_summary

__all__ = ["get_bricks_in_projects", "print_bricks_in_projects", "print_workspace_summary"]



from rich import box
from rich.columns import Columns
from rich.console import Console
from rich.padding import Padding
from rich.table import Table
from rich.theme import Theme

info_theme = Theme(
    {
        "data": "#999966",
        "proj": "#8A2BE2",
        "comp": "#32CD32",
        "base": "#6495ED",
    }
)


def brick_status(brick, bricks) -> str:
    status = ":gear:" if brick in bricks else "-"

    return f"[data]{status}[/]"


def print_diff_details(
    projects_data: list[dict], bases: list[str], components: list[str]
) -> None:
    console = Console(theme=info_theme)
    table = Table(box=box.SIMPLE_HEAD)
    table.add_column("[data]changed brick[/]")

    for project in projects_data:
        project_name = project["name"]
        table.add_column(f"[proj]{project_name}[/]")

    for brick in sorted(components):
        cols = [brick_status(brick, p.get("components")) for p in projects_data]
        table.add_row(f"[comp]{brick}[/]", Columns(cols))

    for brick in sorted(bases):
        cols = [brick_status(brick, p.get("bases")) for p in projects_data]
        table.add_row(f"[base]{brick}[/]", Columns(cols))

    console.print(table, overflow="ellipsis")


def print_detected_changes_in_projects(projects: list[str]) -> None:
    if not projects:
        return

    console = Console(theme=info_theme)

    for project in sorted(projects):
        console.print(f"[data]:gear: Changes found in [/][proj]{project}[/]")


def print_diff_summary(
    tag: str,  bases: list[str], components: list[str]
) -> None:
    console = Console(theme=info_theme)

    console.print(Padding(f"[data]Diff: based on the {tag} tag[/]", (1, 0, 1, 0)))

    if not bases and not components:
        console.print("[data]No brick changes found.[/]")
        return

    if components:
        console.print(f"[comp]Changed components[/]: [data]{len(components)}[/]")

    if bases:
        console.print(f"[base]Changed bases[/]: [data]{len(bases)}[/]")



import logging


def getHandler():
    handler = logging.StreamHandler()
    formatter = logging.Formatter("%(name)s: %(message)s")

    handler.setFormatter(formatter)

    return handler


def getLogger(name="python-polylith") -> logging.Logger:
    logger = logging.getLogger(name)

    if not logger.hasHandlers():
        handler = getHandler()
        logger.addHandler(handler)
        logger.setLevel(logging.INFO)

    return logger



from pathlib import Path

from cleo.helpers import option
from poetry.console.commands.command import Command
from polylith import diff, info, repo


class DiffCommand(Command):
    name = "poly diff"
    description = "Shows changed bricks compared to the latest git tag."

    options = [
        option(
            long_name="short",
            short_name="s",
            description="Print only changed projects",
            flag=True,
        ),
    ]

    def handle(self) -> int:
        root = repo.find_workspace_root(Path.cwd())
        if not root:
            raise ValueError(
                "Didn't find the workspace root. Expected to find a workspace.toml file."
            )

        tag = diff.collect.get_latest_tag(root)

        if not tag:
            self.line("No tags found in repository.")
        else:
            files = diff.collect.get_files(tag)
            bases = diff.collect.get_changed_bases(root, files)
            components = diff.collect.get_changed_components(root, files)
            projects = diff.collect.get_changed_projects(files)
            projects_data = info.get_bricks_in_projects(root)

            short = self.option("short")

            if short:
                self.line(",".join(projects))
            else:
                diff.report.print_diff_summary(tag, bases, components)
                diff.report.print_detected_changes_in_projects(projects)
                diff.report.print_diff_details(projects_data, bases, components)

        return 0



from pathlib import Path

from polylith.dirs import create_dir
from polylith.files import create_file
from polylith.interface import create_interface
from polylith.workspace import parser


def create_brick(
    root: Path, brick: str, namespace: str, package: str, modulename: str = "core"
) -> None:
    dirs_structure = parser.get_brick_structure_from_config(root)
    dirs = dirs_structure.format(brick=brick, namespace=namespace, package=package)
    d = create_dir(root, dirs)

    create_file(d, f"{modulename}.py")
    create_interface(d, namespace, package, modulename)



from collections.abc import Generator
from pathlib import Path

import tomlkit
from polylith.repo import default_toml


def get_project_package_includes(data) -> list[dict]:
    return data["tool"]["poetry"]["packages"]


def get_project_name(data) -> str:
    return data["tool"]["poetry"]["name"]


def get_toml(root: Path) -> tomlkit.TOMLDocument:
    with root.open() as f:
        return tomlkit.loads(f.read())


def get_project_files(root: Path) -> Generator:
    return root.glob(f"projects/**/{default_toml}")


def get_toml_files(root: Path) -> list[tomlkit.TOMLDocument]:
    project_files = get_project_files(root)

    return [get_toml(p) for p in project_files]


def get_project_names(root: Path) -> list[str]:
    tomls = get_toml_files(root)

    return [get_project_name(d) for d in tomls]


def get_packages_for_projects(root: Path) -> list[dict]:
    tomls = get_toml_files(root)

    return [
        {"name": get_project_name(d), "packages": get_project_package_includes(d)}
        for d in tomls
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['polylith',
 'polylith.bricks',
 'polylith.development',
 'polylith.diff',
 'polylith.dirs',
 'polylith.files',
 'polylith.info',
 'polylith.interface',
 'polylith.log',
 'polylith.poetry.commands',
 'polylith.poetry_plugin',
 'polylith.project',
 'polylith.readme',
 'polylith.repo',
 'polylith.test',
 'polylith.workspace']

package_data = \
{'': ['*']}

install_requires = \
['poetry>=1.2,<2.0', 'rich>=12.6.0,<13.0.0', 'tomlkit>=0.11.5,<0.12.0']

entry_points = \
{'poetry.application.plugin': ['poetry-polylith-plugin = '
                               'polylith.poetry_plugin:PolylithPlugin']}

setup_kwargs = {
    'name': 'poetry-polylith-plugin',
    'version': '1.0.1',
    'description': 'A Poetry plugin that adds tooling support for the Polylith Architecture',
    'long_description': '# Poetry Polylith Plugin\n\nThis is a Python `Poetry` plugin, adding CLI support for the Polylith Architecture.\n\n\n## What\'s Polylith?\nFrom the [official docs](https://polylith.gitbook.io/polylith/):\n\n>... Polylith is a software architecture that applies functional thinking at the system scale. It helps us build simple, maintainable, testable, and scalable backend systems. ...\n\nPolylith is an architecture (with tooling support) originally built for Clojure.\nWith this Poetry plugin, Polylith is available in Python too!\n\n### An Architecture well suited for Monorepos\nPolylith is using a components-first architecture. Similar to LEGO, components are building blocks.\nA component can be shared across apps, tools, libraries, serverless functions and services.\n\n\n### Differences between the Clojure & Python implementations\nIn the [official docs](https://polylith.gitbook.io/polylith/) for the Clojure implementation,\nthere is a `interface.clj` file that is used to separate an API from the implementation of a component.\n\nThe Python implementation uses the `__init__.py` to accomplish that. In the Python implementation, the `pyproject.toml` is used to define bases and components.\nIn particular, the `packages` property is used for that.\n\nThis is an example of the _top level_ `pyproject.toml` used when _developing_. This is where you add all bricks (components and bases).\n\n``` shell\n packages = [\n    {include = "development"},\n    {include = "my_namespace/my_component", from = "components"},\n    {include = "my_namespace/my_example_aws_lambda", from = "bases"},\n]\n```\n(using the `loose` theme, see more about that below)\n\nWhen creating a project, the _project specific_ `pyproject.toml` will include all the used components and bases.\nNote that the packages are referenced relative to the project. This is made possible by the Multiproject Poetry plugin.\n\nThis is where you add the bricks used by the actual project.\n\n``` shell\n packages = [\n    {include = "my_namespace/my_component", from = "../../components"},\n    {include = "my_namespace/my_example_aws_lambda", from = "../../bases"},\n]\n```\n\n## Usage\n\n### Install Poetry & plugins\nWith the `Poetry` version 1.2 or later installed, you can add plugins.\n\nAdd the [Multiproject](https://github.com/DavidVujic/poetry-multiproject-plugin) plugin, that will enable the very important __workspace__ support to Poetry.\n``` shell\npoetry self add poetry-multiproject-plugin\n```\n\nAdd the Polylith plugin:\n``` shell\npoetry self add poetry-polylith-plugin\n```\n\n### Create a repository\nCreate a directory for your code, initialize it with __git__ and create a basic __Poetry__ setup:\n\n``` shell\ngit init\n\npoetry init\n```\n\n### Commands\nThe `create workspace` command will create a Polylith workspace, with a basic Polylith folder structure.\n\n\n#### Create\n``` shell\npoetry poly create workspace --name my_namespace --theme <tdd or loose>\n```\n\n*New:* `theme` is a new Python Polylith feature and defines what kind of component structure - or theme - to use.\n\n`tdd` is the default and will set the structure according to the original Polylith Clojure implementation, such as:\n`components/<package>/src/<namespace>/<package>` with a corresponding `test` folder.\n\n`loose` is a new theme, for a more familiar structure for Python:\n`components/<namespace>/<package>` and will put a `test` folder at the root of the repository.\n\n\nAdd a component:\n\n``` shell\n# This command will create a component - i.e. a Python package in a namespaced folder.\npoetry poly create component --name my_component\n```\n\nAdd a base:\n\n``` shell\n# This command will create a base - i.e. a Python package in a namespaced folder.\npoetry poly create base --name my_example_aws_lambda\n```\n\nAdd a project:\n\n``` shell\n# This command will create a project - i.e. a pyproject.toml in a project folder. No code in this folder.\npoetry poly create project --name my_example_aws_lambada_project\n```\n\n#### Info\nShow info about the workspace:\n\n``` shell\npoetry poly info\n```\n\nShows what has changed since the most recent stable point in time:\n\n``` shell\npoetry poly diff\n```\n\nThe `diff` command will compare the current state of the repository, compared to a `git tag`.\nThe tool will look for the latest tag according to a certain pattern, such as `stable-*`.\nThe pattern can be configured in `workspace.toml`.\n\nThe `diff` command is useful in a CI environment, to determine if a project should be deployed or not.\nThe command has a `--short` flag to only print a comma separated list of changed projects to the standard output.\n\n\nUseful for CI:\n``` shell\npoetry poly diff --short\n```\n\n#### Testing\nThe `create` commands will also create corresponding unit tests. It is possible to disable thi behaviour\nby setting `enabled = false` in the `workspace.toml` file.\n\n\n#### Workspace configuration\nAn example of a workspace configuration:\n\n``` shell\n[tool.polylith]\nnamespace = "my_namespace"\ngit_tag_pattern = "stable-*"\n\n[tool.polylith.structure]\ntheme = "loose"\n\n[tool.polylith.test]\nenabled = true\n```\n',
    'author': 'David Vujic',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/davidvujic/python-polylith',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
